/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.constants;

import javax.swing.ImageIcon;

/**
 * Class Description:
 * 		Application constants
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class ApplicationConstants {

	/** 
	 * To make sure noone is using this constructor.
	 * Else the compiler would create a public one.
	 * */
	private ApplicationConstants() {}
	
	/**
	 * GLOBAL
	 * For the whole Application
	 */
	
	/* Main Properties */
	public static final String GLOBAL_PROPS = "./viergewinnt.properties";
	
	// The main frame title
	public static String FRAME_TITLE = "VierGewinnt";
	
	// The version of the program
	public static String VERSION = "1.2 - 07/20/2006";
	
	// The size of the playfield
	public static int PLAYFIELD_SIZE_COLS = 7;
	public static int PLAYFIELD_SIZE_ROWS = 6;
	public static int PLAYFIELD_WIN_SIZE = 4;
	
	// The players
	public static int PLAYER_HUMAN = 1;
	public static int PLAYER_CPU = 2;
	public static int PLAYER_EXTERNAL = 3;
	
	// The player representations within the DAT file
	public static int FILE_PLAYER_EMPTY = 0;
	public static int FILE_PLAYER_1 = 1;
	public static int FILE_PLAYER_2 = 2;
	
	// The icons for the playfield
	public static ImageIcon PLAYFIELD_STATE_EMPTY = new ImageIcon("images/state_empty.png");
	public static ImageIcon PLAYFIELD_STATE_GREEN = new ImageIcon("images/state_green.png");
	public static ImageIcon PLAYFIELD_STATE_RED = new ImageIcon("images/state_red.png");
	public static ImageIcon PLAYFIELD_ARROW_GREEN = new ImageIcon("images/arrow_green.png");
	public static ImageIcon PLAYFIELD_ARROW_RED = new ImageIcon("images/arrow_red.png");
	public static ImageIcon PLAYFIELD_ARROW_OVER = new ImageIcon("images/arrow_over.png");
	
	public static int PLAYFIELD_STATE_ICON_WIDTH = 80;
	public static int PLAYFIELD_STATE_ICON_HEIGHT = 86;
	
	public static int WINDOW_MARGIN_TOP = 80;
	
	/**
	 * Retrieve the startup text for the application
	 * @return the startup text
	 */
	public static String getStartupText() {
	    
	    // The startup text string
	    String text = "";
	    
	    // Build the startup text string
	    text = "################################\n" + 
	    		"Entscheidungstheorie\n" +
	    		"Prof. Dr. Peters - SS2006\n" +
	    		"FH Muenchen - Master of Science\n" +
	    		"VierGewinnt - Version " + ApplicationConstants.VERSION + "\n" + "################################\n" +
	    				"Written by ...\n" +
	    				"Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net) \n" +
	    				"################################\n";
	    
	    return text;
	}
	
	/**
	 * Retrieve the help text for the application
	 * @return the help text
	 */
	public static String getHelpText() {
	    
	    // The help text string
	    String text = "";
	    
	    // Build the help text string
	    text = "\nHelp: \nYou need to provide the following command-line arguments: \n start.bat <Columns> <Rows> <WinSize> <Player1Mode> <Player2Mode> <Additional Player1Info> <Additional Player2Info>";
	    text += "\n";
	    text += "\n <Columns> \t\t The number of colums of the playfield (i.e. 7)";
	    text += "\n <Rows> \t\t\t The number of rows of the playfield (i.e. 6)";
	    text += "\n <WinSize> \t\t\t The number of stones the player needs to have in a row/column/diagonal to win (i.e. 4)";
	    text += "\n <Player1Mode>	\t Mode for player 1 (see below)";
	    text += "\n <Player2Mode> \t Mode for player 2 (see below)";
	    text += "\n <Player1Info> \t\t Additional information for player 1 (see below)";
	    text += "\n <Player2Info> \t\t Additional information for player 2 (see below)";
	    text += "\n";
	    text += "\n Player Modes: {c,h,e}";
	    text += "\n c \t CPU player (needs additional player information)";
	    text += "\n h \t Human player";
	    text += "\n e \t External player (i.e. via DAT file) (needs additional player information)";
	    text += "\n";
	    text += "\n Additional Player Information: ";
	    text += "\n CPU player \t Tree search depth needs to be specified (>= 1) (integer)";
	    text += "\n External player \t DAT File needs to be specified (path to file)";
	    text += "\n";
	    text += "\n Examples: ";
	    text += "\n start.bat 7 6 4 h h \t\t\t (Human vs. Human)";
	    text += "\n start.bat 7 6 4 h c 2 \t\t\t (Human vs. CPU (Level 2))";
	    text += "\n start.bat 7 6 4 c h 2 \t\t\t (CPU (Level 2) vs. Human)";
	    text += "\n start.bat 7 6 4 c c 2 5 \t\t (CPU (Level 2) vs. CPU (Level 5))";
	    text += "\n start.bat 7 6 4 h e test.dat \t (Human vs. External (test.dat))";
	    text += "\n start.bat 7 6 4 c e 2 test.dat \t (CPU (Level 2) vs. External (test.dat))";
	    text += "\n start.bat 7 6 4 e c test.dat 2 \t (External (test.dat) vs. CPU (Level 2))";
	    text += "\n";
	    text += "\n Examples: External Games";
	    text += "\n (on client 1) start.bat 7 6 4 h e test.dat";
	    text += "\n (on client 2) start.bat 7 6 4 e h test.dat";
	    text += "\n (Human vs. Human)";
	    text += "\n";
	    text += "\n (on client 1) start.bat 7 6 4 c e 2 test.dat";
	    text += "\n (on client 2) start.bat 7 6 4 e h test.dat";
	    text += "\n (CPU (Level 2) vs. Human)";
	    
	    return text;
	}
}
