/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.objects;

import net.woellert.viergewinnt.VierGewinntStartup;
import net.woellert.viergewinnt.objects.Playfield.SlotState;
import net.woellert.viergewinnt.player.AbstractPlayer;
import net.woellert.viergewinnt.player.CPUPlayer;
import net.woellert.viergewinnt.util.logging.LoggingManager;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * Class Description:
 * 		Main Game Engine
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class Game {

	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(Game.class.getName());

	// The playfield
	protected Playfield field = null;

	// The Players
	protected AbstractPlayer player1 = null;
	protected AbstractPlayer player2 = null;

	/**
	 * Constructor
	 * @param player1 the first player
	 * @param player2 the second player
	 * @param field the playfield
	 */
	public Game(AbstractPlayer player1, AbstractPlayer player2, Playfield field) {
		this.field = field;
		this.player1 = player1;
		this.player2 = player2;
	}

	/**
	 * Run the game
	 */
	public void run() {

		// Run, start with player 1		
		while (true) {

			// Player 1 Turn
			if (localLogger.isEnabledFor(Level.INFO)) {
				localLogger.log(Level.INFO, "Player 1 - Your turn ...");
			}

			// Tell player1 to do his/her turn
			VierGewinntStartup.currentPlayer(1);
			player1.doTurn(field, SlotState.PLAYER_1);

			// Check for game over
			int winner = field.isGameOver();

			if (winner == 1 || winner == 2) {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "##########################");
					localLogger.log(Level.INFO, "WINNER ! Player " + winner);
					localLogger.log(Level.INFO, "##########################");
				}
				VierGewinntStartup.gameIsOver(winner);
				break;

			} else if (winner == -1) {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "##########################");
					localLogger.log(Level.INFO, "No WINNER ! Even !");
					localLogger.log(Level.INFO, "##########################");
				}
				VierGewinntStartup.gameIsOver(0);
				break;
			}
			
			// Check if we should wait a short time because both players are CPUs
			if(player1 instanceof CPUPlayer && player2 instanceof CPUPlayer) {
				try {
					Thread.sleep(400);
				} catch (InterruptedException e) {}
			}

			// Player 2 Turn
			if (localLogger.isEnabledFor(Level.INFO)) {
				localLogger.log(Level.INFO, "Player 2 - Your turn ...");
			}
			
			VierGewinntStartup.currentPlayer(2);
			player2.doTurn(field, SlotState.PLAYER_2);

			// Check for game over
			winner = field.isGameOver();

			if (winner == 1 || winner == 2) {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "##########################");
					localLogger.log(Level.INFO, "WINNER ! Player " + winner);
					localLogger.log(Level.INFO, "##########################");
				}
				VierGewinntStartup.gameIsOver(winner);
				break;

			} else if (winner == -1) {
				if (localLogger.isEnabledFor(Level.INFO)) {
					localLogger.log(Level.INFO, "##########################");
					localLogger.log(Level.INFO, "No WINNER ! Even !");
					localLogger.log(Level.INFO, "##########################");
				}
				VierGewinntStartup.gameIsOver(0);
				break;
			}
			
			// Check if we should wait a short time because both players are CPUs
			if(player1 instanceof CPUPlayer && player2 instanceof CPUPlayer) {
				try {
					Thread.sleep(400);
				} catch (InterruptedException e) {}
			}
		}
	}
}
