/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.objects;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.geom.AffineTransform;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import net.woellert.viergewinnt.VierGewinntStartup;
import net.woellert.viergewinnt.constants.ApplicationConstants;
import net.woellert.viergewinnt.objects.Playfield.SlotState;
import net.woellert.viergewinnt.player.AbstractPlayer;
import net.woellert.viergewinnt.player.HumanPlayer;

/**
 * Class Description:
 * 		The frame displaying the playfield and handling HumanPlayer mouse input events
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class PlayfieldFrame extends JFrame implements WindowListener, MouseMotionListener {

	// The displayed playfield
	protected Playfield field = null;

	// The panel displaying the playfield
	protected PlayfieldPanel fieldPanel = null;

	/**
	 * Constructor
	 * @param field the playfield to display
	 */
	public PlayfieldFrame(Playfield field) {
		super(ApplicationConstants.FRAME_TITLE);
		this.field = field;

		// Add windowlistener and mouselistener
		this.addWindowListener(this);
		this.addMouseMotionListener(this);
		
		// Set the frame size
		this.setSize(ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH * ApplicationConstants.PLAYFIELD_SIZE_COLS + 10, 
				ApplicationConstants.WINDOW_MARGIN_TOP +  ApplicationConstants.PLAYFIELD_STATE_ICON_HEIGHT * ApplicationConstants.PLAYFIELD_SIZE_ROWS + 28);
		
		// Exit the application when closing the frame
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Set the basic layout and add the panel to display the playfield
		this.getContentPane().setLayout(new BorderLayout());
		this.fieldPanel = new PlayfieldPanel(field);
		this.getContentPane().add(fieldPanel, BorderLayout.CENTER);
	}
	
	/**
	 * WINDOW LISTENER INTERFACE METHODS
	 */

	/**
	 * Called when the window is closing
	 * @param arg0
	 */
	public void windowClosing(WindowEvent arg0) {
		System.exit(1);
	}

	public void windowDeactivated(WindowEvent arg0) {}
	public void windowDeiconified(WindowEvent arg0) {}
	public void windowIconified(WindowEvent arg0) {}
	public void windowOpened(WindowEvent arg0) {}
	public void windowActivated(WindowEvent arg0) {}
	public void windowClosed(WindowEvent arg0) {}

	/**
	 * MOUSE MOTION LISTENER INTERFACE METHODS
	 */

	/**
	 * Called when the mouse is moved
	 * @param e
	 */
	public void mouseMoved(MouseEvent e) {

		// Grab the coordiantes and determine the column
		Point p = e.getPoint();
		int col = (int) (p.getX() / (this.getSize().getWidth() / ApplicationConstants.PLAYFIELD_SIZE_COLS));
		
		// Check if the arrow should be displayed for the current player (only show it for Human Players)
		AbstractPlayer currentPlayer = VierGewinntStartup.getCurrentPlayer();
		
		if(currentPlayer != null) {
			
			if(currentPlayer instanceof HumanPlayer) {
				this.fieldPanel.displayPlayerArrow(col);
			} else {
				this.fieldPanel.displayPlayerArrow(-1);
			}
			
		} else {
			
			// Check if we have a win or even state and need to display the RED-X arrow
			if(VierGewinntStartup.CURRENT_PLAYER == 0) {
				// WinState, display the arrows
				this.fieldPanel.displayPlayerArrow(col);
			} else {
				// No player, so no arrows to display
				this.fieldPanel.displayPlayerArrow(-1);
			}
		}
	}
	
	public void mouseDragged(MouseEvent arg0) {}
		
	/**
	 * Class Description:
	 * 		The panel displaying the playfield
	 * 
	 * @author Thomas Woellert (thomas@woellert.net)
	 */
	class PlayfieldPanel extends JPanel implements PlayfieldChangeListener {

		// The displayed playfield
		protected Playfield field = null;
		
		// The column for the player arrow
		protected int playerArrowCol = -1;
		
		/**
		 * Constructor
		 * @param field the playfield to display
		 */
		public PlayfieldPanel(Playfield field) {
			super();
			
			this.field = field;
			this.field.addChangeListener(this);
		}

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			
			// Grab the 2D graphics context
			Graphics2D g2 = (Graphics2D)g;
			
			// Clear the area
			g2.setColor(Color.WHITE);
			g2.fillRect(0, 0, this.getWidth(), this.getHeight());
			
			// Apply a proper affine transform in case the frame is bigger than the area we need to draw into
			int neededPixelWidth = ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH * field.getCols();
			int neededPixelHeight = ApplicationConstants.PLAYFIELD_STATE_ICON_HEIGHT * field.getRows() + ApplicationConstants.WINDOW_MARGIN_TOP;
						
			AffineTransform trans = new AffineTransform();
			double xScale = (double) (this.getWidth()) / (double) (neededPixelWidth);
			double yScale = (double) (this.getHeight()) / (double) (neededPixelHeight);
			trans.scale(xScale, yScale);
			g2.setTransform(trans);
			
			// Run through the whole playfield
			for(int i = 0; i < field.getRows(); i++) {
				for(int j = 0; j < field.getCols(); j++) {
					
					// Get the state and determine the icon to represent it
					SlotState state = field.getState(i, j);
					ImageIcon icon = null;
					
					switch(state) {
						case EMPTY:
							icon = ApplicationConstants.PLAYFIELD_STATE_EMPTY;
							break;
							
						case PLAYER_1:
							icon = ApplicationConstants.PLAYFIELD_STATE_GREEN;
							break;
							
						case PLAYER_2:
							icon = ApplicationConstants.PLAYFIELD_STATE_RED;
							break;
						
						default:
							System.err.println("ERROR");
					}
					
					// Determine the coordinates to draw the icon at
					int iconX = (j) * ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH;
					int iconY = ((ApplicationConstants.PLAYFIELD_SIZE_ROWS-i-1) * ApplicationConstants.PLAYFIELD_STATE_ICON_HEIGHT) + ApplicationConstants.WINDOW_MARGIN_TOP;
					
					// Draw the icon
					g2.drawImage(icon.getImage(), iconX, iconY, null);
				}
			}
			
			// Check if we need to display player arrows
			AbstractPlayer currentPlayer = VierGewinntStartup.getCurrentPlayer();
			if(VierGewinntStartup.CURRENT_PLAYER == 1 && currentPlayer != null) {
				
				// Player 1 
				if(playerArrowCol != -1 && currentPlayer instanceof HumanPlayer) {
					ImageIcon arrowIcon = ApplicationConstants.PLAYFIELD_ARROW_GREEN;		
					int iconX = playerArrowCol * ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH;
					int iconY = 0;
					g2.drawImage(arrowIcon.getImage(), iconX, iconY, null);
				}
				
			} else if(VierGewinntStartup.CURRENT_PLAYER == 2 && currentPlayer != null) {
				
				// Player 2
				if(playerArrowCol != -1 && currentPlayer instanceof HumanPlayer) {
					ImageIcon arrowIcon = ApplicationConstants.PLAYFIELD_ARROW_RED;
					int iconX = playerArrowCol * ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH;
					int iconY = 0;
					g2.drawImage(arrowIcon.getImage(), iconX, iconY, null);
				}
				
			} else if(VierGewinntStartup.CURRENT_PLAYER == 0) {
				
				// No player, game is over
				if(playerArrowCol != -1) {
					ImageIcon arrowIcon = ApplicationConstants.PLAYFIELD_ARROW_OVER;
					int iconX = playerArrowCol * ApplicationConstants.PLAYFIELD_STATE_ICON_WIDTH;
					int iconY = 0;
					g2.drawImage(arrowIcon.getImage(), iconX, iconY, null);
				}
			}
		}
		
		/**
		 * Display the arrow pointing at the given column
		 * @param col the column to point at
		 */
		public void displayPlayerArrow(int col) {
			playerArrowCol = col;
			this.repaint();
		}

		/**
		 * PLAYFIELD CHANGE LISTENER INTERFACE METHOD
		 * The playfield changed which we added us as change listener to
		 * @param field
		 */
		public void playfieldChanged(Playfield field) {
			this.repaint();
		}
	}
}
