/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.player;

import java.io.File;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import net.woellert.viergewinnt.objects.Playfield;
import net.woellert.viergewinnt.objects.Playfield.SlotState;
import net.woellert.viergewinnt.util.io.DATFileLoader;
import net.woellert.viergewinnt.util.io.DATFileSaver;
import net.woellert.viergewinnt.util.logging.LoggingManager;

/**
 * Class Description:
 * 		External player implementation communicating via a DAT file
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class ExternalPlayer extends AbstractPlayer {
	
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(ExternalPlayer.class.getName());
	
	// The used dat file
	protected File datFile = null;

	/**
	 * Constructor
	 * @param datFile the dat file to use
	 */
	public ExternalPlayer(File datFile) {
		super();
		this.datFile = datFile;
	}

	@Override
        public void doTurn(Playfield field, SlotState playerState) {
	        
		// Write the current playfield information to the dat file
		if(!DATFileSaver.save(datFile, field)) {			
			if (localLogger.isEnabledFor(Level.ERROR)) {
				localLogger.log(Level.ERROR, "Error while saving DAT file: " + datFile.getName());
			}
		}
		
		// Remember the time the dat file has been saved
		long lastModifiedTime = datFile.lastModified();
		
		if (localLogger.isEnabledFor(Level.INFO)) {
			localLogger.log(Level.INFO, "DAT File saved. Last modified: " + lastModifiedTime);
			localLogger.log(Level.INFO, "Waiting for other players turn ...");
		}
		
		// Wait till the external player completed its turn
		while(lastModifiedTime == datFile.lastModified()) {
			try {
	                        Thread.sleep(100);
                        } catch (InterruptedException e) {
	                        e.printStackTrace();
                        }
		}
		
		if (localLogger.isEnabledFor(Level.INFO)) {
			localLogger.log(Level.INFO, "... other player completed turn.");
		}
		
		// Load the new playfield information from the dat file
		Playfield newField = DATFileLoader.load(datFile);
		
		if(newField == null) {
			if (localLogger.isEnabledFor(Level.ERROR)) {
				localLogger.log(Level.ERROR, "Error while loading DAT file: " + datFile.getName());
			}
		}

		// Fill the original playfield with the new state
		field.newPlayField(newField);
        }
}
