/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.util.io;

import java.io.File;
import java.io.IOException;
import java.util.List;

import net.woellert.viergewinnt.constants.ApplicationConstants;
import net.woellert.viergewinnt.objects.Playfield;
import net.woellert.viergewinnt.util.logging.LoggingManager;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * Class Description:
 * 		Loader used by the External Player to read DAT files
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class DATFileLoader {

	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(DATFileLoader.class.getName());

	/**
	 * Only static methods, therefore invisible constructor
	 */
	protected DATFileLoader() {}

	/**
	 * Load the given dat file and return the playfield it represents
	 * @param datFile the file to load
	 * @return the playfield based on the files' contents
	 */
	public static Playfield load(File datFile) {

		// Create an empty playfield
		Playfield field = new Playfield(ApplicationConstants.PLAYFIELD_SIZE_ROWS, ApplicationConstants.PLAYFIELD_SIZE_COLS);

		try {
			// Read all lines from the file
			List<String> fileContent = FileUtils.readLines(datFile, null);

			// Go through all lines
			for (int i = 0; i < fileContent.size(); i++) {
				String currentLine = fileContent.get(i);

				// Split the line
				String[] currentContents = currentLine.split("\\s+");

				// Check for proper length
				if (currentContents.length != ApplicationConstants.PLAYFIELD_SIZE_COLS) {
					if (localLogger.isEnabledFor(Level.ERROR)) {
						localLogger.log(Level.ERROR, "Contents in line do not match:" + currentContents.length);
						localLogger.log(Level.ERROR, currentLine);
						return null;
					}
				}

				// Read all data from the current line
				for (int col = 0; col < ApplicationConstants.PLAYFIELD_SIZE_COLS; col++) {
					int currentFieldEntry = Integer.parseInt(currentContents[col]);

					if (currentFieldEntry == ApplicationConstants.FILE_PLAYER_EMPTY) {
						field.setState((ApplicationConstants.PLAYFIELD_SIZE_ROWS-i-1), col, Playfield.SlotState.EMPTY);
					} else if (currentFieldEntry == ApplicationConstants.FILE_PLAYER_1) {
						field.setState((ApplicationConstants.PLAYFIELD_SIZE_ROWS-i-1), col, Playfield.SlotState.PLAYER_1);
					} else if (currentFieldEntry == ApplicationConstants.FILE_PLAYER_2) {
						field.setState((ApplicationConstants.PLAYFIELD_SIZE_ROWS-i-1), col, Playfield.SlotState.PLAYER_2);
					}
				}
			}

		} catch (IOException e) {
			if (localLogger.isEnabledFor(Level.ERROR)) {
				localLogger.log(Level.ERROR, "Error while loading the log file !", e);
				return null;
			}
		}

		return field;
	}
}
