/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.util.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;

import net.woellert.viergewinnt.objects.Playfield;
import net.woellert.viergewinnt.util.logging.LoggingManager;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * Class Description:
 * 		Saver used by the External Player to write DAT files
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class DATFileSaver {

	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(DATFileSaver.class.getName());

	/**
	 * Only static methods, therefore invisible constructor
	 */
	protected DATFileSaver() {}

	/**
	 * Save the given playfield to the DAT file
	 * @param datFile the file to save to
	 * @param field the playfield
	 * @return true if all went well, false otherwise
	 */
	public static boolean save(File datFile, Playfield field) {

		try {
			// Open the file output stream
			FileOutputStream outputStream = new FileOutputStream(datFile, false);

			// Create a new StringWriter
			StringWriter writer = new StringWriter();

			// Go through all playfield positions
			for (int row = (field.getRows()-1); row >= 0; row--) {
				for (int col = 0; col < field.getCols(); col++) {

					Playfield.SlotState currentState = field.getState(row, col);

					switch (currentState) {
					case EMPTY:
						writer.write('0');
						writer.write(' ');
						break;

					case PLAYER_1:
						writer.write('1');
						writer.write(' ');
						break;

					case PLAYER_2:
						writer.write('2');
						writer.write(' ');
						break;

					default:
						if (localLogger.isEnabledFor(Level.ERROR)) {
							localLogger.log(Level.ERROR, "Error while saving the dat file ! Unknown playfield state: " + currentState);
						}
						return false;
					}
				}

				// New line
				writer.write('\n');
			}

			// Grab the buffer from the writer
			StringBuffer buffer = writer.getBuffer();
			String string = buffer.toString();

			// Create an OutputStreamWriter
			OutputStreamWriter outputStreamWriter = new OutputStreamWriter(outputStream);

			// Write the String to the output stream and close it
			outputStreamWriter.write(string);
			outputStreamWriter.close();

		} catch (Exception e) {
			if (localLogger.isEnabledFor(Level.ERROR)) {
				localLogger.log(Level.ERROR, "Saving file failed: " + datFile.getName(), e);
			}
		}
		
		return true;
	}
}
